@@ -17,6 +17,7 @@ from .action_processor import get_actions, list_actions
 from .aliases import AliasManager
 from .api import get_soco_object, run_command
 from .cmd_parser import CLIParser
+from .keystroke_capture import get_keystroke
 from .utils import (
     RewindableList,
     get_readline_history,
@@ -28,17 +29,20 @@ from .utils import (
 )
 from .wait_actions import process_wait
 
-
 # Alias Manager
 am = AliasManager()
 
 
-def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
+def interactive_loop(
+    speaker_name, use_local_speaker_list=False, no_env=False, single_keystroke=False
+):
+
+    speaker = None
+    saved_speaker = None
+    pushed = False
 
     # Is the speaker name set on the command line?
     # Note: ignores SPKR set as part of the environment
-    speaker = None
-    saved_speaker = None
     if speaker_name:
         speaker, error_msg = get_soco_object(
             speaker_name, use_local_speaker_list=use_local_speaker_list
@@ -48,7 +52,10 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
             speaker_name = None
 
     print("\nEntering SoCo-CLI interactive shell.")
-    print("Type 'help' for available shell commands.\n")
+    if single_keystroke:
+        print("Single Keystroke Mode ... 'x' to exit.\n")
+    else:
+        print("Type 'help' for available shell commands.\n")
     if not RL:
         print("Note: Autocompletion not currently available on Windows.\n")
 
@@ -62,12 +69,27 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
         readline.set_completer_delims(" ")
         _get_readline_history()
 
+    single_keystroke = single_keystroke
+
     # Input loop
     while True:
         if speaker_name and speaker:
-            command_line = input("SoCo-CLI [{}] > ".format(speaker.player_name))
+            prompt = "SoCo-CLI [{}] > ".format(speaker.player_name)
         else:
-            command_line = input("SoCo-CLI [] > ")
+            prompt = "SoCo-CLI [] > "
+
+        if single_keystroke:
+            command_line = get_keystroke()
+            if speaker_name and speaker:
+                speaker_text = "[{}]".format(speaker.player_name)
+            else:
+                speaker_text = "[]"
+            print("{} Command =".format(speaker_text), command_line)
+            if command_line == "x":
+                single_keystroke = False
+                continue
+        else:
+            command_line = input(prompt)
 
         if command_line == "":
             continue
@@ -89,25 +111,13 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
             except IndexError:
                 break
 
-            # Replace the command sequence with the contents of an alias
-            # This is tricky for multiple sequences
+            # Is this an alias?
             if command[0] in am.alias_names():
-                action = am.action(command[0])
-                actions = shlex_split(action)
-                cli_parser.parse(actions)
-                new_sequences = cli_parser.get_sequences()
-                logging.info("Sequences in the alias: {}".format(new_sequences))
-                command = new_sequences.pop(0)
-                # Multiple sequences?
-                if len(new_sequences):
-                    # Insert the additional sequences in the list
-                    index = command_sequences.index()
-                    for sequence in new_sequences:
-                        logging.info(
-                            "Inserting new sequence {} at {}".format(sequence, index)
-                        )
-                        command_sequences.insert(index, sequence)
-                        index += 1
+                ap = AliasProcessor()
+                index = command_sequences.index()
+                ap.process(command[0], am, command_sequences)
+                command_sequences.rewind_to(index)
+                continue
 
             if command[0] == "0":
                 # Unset the active speaker
@@ -131,6 +141,14 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                 _show_actions()
                 continue
 
+            if command_lower in ["single-keystroke", "sk"]:
+                if RL:  # Use as a proxy for 'not Windows'
+                    print("Single keystroke mode ... 'x' to exit")
+                    single_keystroke = True
+                else:
+                    print("Single keystroke mode is not supported on Windows")
+                continue
+
             if command_lower == "speakers":
                 _print_speaker_list(use_local_speaker_list=use_local_speaker_list)
                 continue
@@ -183,6 +201,10 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                 continue
 
             if command_lower == "push":
+                if pushed == True:
+                    print("Active speaker already pushed ... ignored")
+                    continue
+                pushed = True
                 if speaker:
                     logging.info(
                         "Pushing current active speaker: {}".format(speaker.player_name)
@@ -195,14 +217,22 @@ def interactive_loop(speaker_name, use_local_speaker_list=False, no_env=False):
                 continue
 
             if command_lower == "pop":
+                if pushed == False:
+                    print("No active speaker state to pop ... ignored")
+                    continue
                 logging.info("Popping the saved speaker state")
                 if saved_speaker:
                     speaker = saved_speaker
                     speaker_name = speaker.player_name
                     logging.info("Saved speaker = '{}'".format(speaker_name))
                     saved_speaker = None
+                elif pushed:
+                    saved_speaker = None
+                    speaker = None
+                    speaker_name = None
                 else:
                     logging.info("No saved speaker")
+                pushed = False
                 continue
 
             # Alias creation, update, and deletion
@@ -324,6 +354,8 @@ COMMANDS = [
     "push",
     "rescan",
     "set ",
+    "single-keystroke",
+    "sk",
     "speakers",
 ]
 
@@ -394,7 +426,9 @@ This is SoCo-CLI interactive mode. Interactive commands are as follows:
                     'set "Front Reception"'. Unambiguous partial, case-insensitive
                     matches are supported, e.g., 'set front'.
                     To unset the active speaker, omit the speaker name,
-                    or just enter '0'.   
+                    or just enter '0'.
+    'sk'        :   Enters 'single keystroke' mode. (Also 'single-keystroke'.)
+                    (Not supported on Windows).
     'speakers'  :   List the names of all available speakers
     
     The command syntax is the same as when using 'sonos' from the command line.
@@ -440,3 +474,44 @@ def _restore_quotes(command):
     for index, parts in enumerate(command):
         if len(parts.split()) > 1:
             command[index] = '"' + parts + '"'
+
+
+class AliasProcessor:
+    def __init__(self):
+        self._used_aliases = []
+
+    def process(self, alias_name, am, command_list):
+
+        if alias_name in self._used_aliases:
+            print("Error: Alias loop detected ... stopping")
+            return False
+        else:
+            self._used_aliases.append(alias_name)
+
+        alias_actions = am.action(alias_name)
+        action_elements = shlex_split(alias_actions)
+
+        cli_parser = CLIParser()
+        cli_parser.parse(action_elements)
+        sequences = cli_parser.get_sequences()
+
+        logging.info("Unpacking the alias '{} -> '{}'".format(alias_name, sequences))
+
+        index = command_list.index()
+        for sequence in sequences:
+            # Recurse if the sequence is itself an alias
+            if sequence[0] in am.alias_names():
+                logging.info("Recursively unpacking the alias '{}'".format(sequence[0]))
+                if self.process(sequence[0], am, command_list):
+                    index = command_list.index()
+                else:
+                    return False
+
+            # Not an alias, so insert the sequence in the command list
+            # at the correct index, and increment the index
+            else:
+                logging.info("Inserting new sequence {} at {}".format(sequence, index))
+                command_list.insert(index, sequence)
+                index += 1
+
+        return True
